/*
 * This file is part of the Nepomuk KDE project.
 * Copyright (C) 2008 Adrien Bustany <madcat@mymadcat.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#ifndef SYNCANNOTATIONPLUGIN_H
#define SYNCANNOTATIONPLUGIN_H

#include <QtCore>
#include "annotationplugin.h"

namespace Nepomuk {

/**
 * \class SyncAnnotationPlugin syncannotationplugin.h SyncAnnotationPlugin
 *
 * \brief Additionnal layer to use an AnnotationPlugin is a synchronous way (ie all results are ready when you get them)
 *
 * This class is thread safe, but the functions are protected by a lock preventing
 * concurrent executions.
 *
 * \author Adrien Bustany <madcat@mymadcat.com>
 */
class NEPOMUK_EXPORT SyncAnnotationPlugin : public QObject
{

public:
	SyncAnnotationPlugin(AnnotationPlugin* plugin);
	~SyncAnnotationPlugin();

	QString name() const;
	QString iconName() const;

	/**
	 * Retrieves all the resources related with a given keyword
	 */
	AnnotationResult* getResources(const QString& filter);

	/**
	 * Retrieves all the resources related with the \a filter keyword, and having the given \a property
	 */
	AnnotationResult* getResources(const QString& filter, const Types::Property& property, const Soprano::Node& object);

	/**
	 * Retrieves the resource with the given \p resourceUri
	 */
	AnnotationResult* getResource(const QUrl& resourceUri);

	/**
	 * Retrieves all the relevant relations for the given mimetype as a list of <Property, Relevance> tuples
	 */
	QList<AnnotationProperty> getProperties(const QString& mimetype);

	/**
	 * Retrieves all the relevant relations for the given RDF type as a list of <Property, Relevance> tuples
	 */
	QList<AnnotationProperty> getProperties(const QUrl& rdfType);

	/**
	 * Effectively associates the file at the given url with the relation identified by the uri
	 */
	void addRelation(const Types::Property& property, const QUrl& subject, const Soprano::Node& object);
	void addRelation(const AnnotationProperty& property, const QUrl& subject, const Soprano::Node& object);

private:
	class Private;
	Private* d;
};

} // End namespace Nepomuk

#endif
